==> cfengine3_3.15.2.orig.tar.gz: cfengine-3.15.2/cf-check/cf-check.c <==
#include <lmdump.h>
#include <dump.h>
#include <diagnose.h>
#include <backup.h>
#include <repair.h>
#include <string_lib.h>
#include <logging.h>
#include <man.h>

static void print_version()
{
    printf("cf-check BETA version %s\n", VERSION);
}

static void print_help()
{
    printf(
        "\n"
        "cf-check:\n"
        "\tUtility for diagnosis and repair of local CFEngine databases.\n"
        "\tThis BETA version of the tool is for testing purposes only.\n"
        "\n"
        "Commands:\n"
        "\tdump - Print the contents of a database file\n"
        "\tdiagnose - Assess the health of one or more database files\n"
        "\tbackup - Copy database files to a timestamped folder\n"
        "\trepair - Diagnose, then backup and delete any corrupt databases\n"
        "\tversion - Print version information\n"
        "\thelp - Print this help menu\n"
        "\n"
        "Usage:\n"
        "\t$ cf-check <command> [options] [file ...]\n"
        "\n"
        "Examples:\n"
        "\t$ cf-check dump " WORKDIR "/state/cf_lastseen.lmdb\n"
        "\t$ cf-check lmdump -a " WORKDIR "/state/cf_lastseen.lmdb\n"
        "\t$ cf-check diagnose\n"
        "\t$ cf-check repair\n"
        "\n");
}

static const char *const CF_CHECK_SHORT_DESCRIPTION =
    "Utility for diagnosis and repair of local CFEngine databases.";

static const char *const CF_CHECK_MANPAGE_LONG_DESCRIPTION =
    "cf-check does not evaluate policy or rely on the integrity of the\n"
    "databases. It is intended to be able to detect and repair a corrupt\n"
    "database.";

static const Description COMMANDS[] =
{
    {"help",     "Prints general help or per topic",
                 "cf-check help [command]"},
    {"diagnose", "Assess the health of one or more database files",
                 "cf-check diagnose"},
    {"backup",   "Backup database files to a timestamped folder",
                 "cf-check backup"},
    {"repair",   "Diagnose, then backup and delete any corrupt databases",
                 "cf-check repair"},
    {"dump",     "Print the contents of a database file",
                 "cf-check dump " WORKDIR "/state/cf_lastseen.lmdb"},
    {"lmdump",   "LMDB database dumper (deprecated)",
                 "cf-check lmdump -a " WORKDIR "/state/cf_lastseen.lmdb"},
    {NULL, NULL, NULL}
};

static const struct option OPTIONS[] =
{
    {"help",        optional_argument,  0, 'h'},
    {"manpage",     no_argument,        0, 'M'},
    {"version",     no_argument,        0, 'V'},
    {"debug",       no_argument,        0, 'd'},
    {"verbose",     no_argument,        0, 'v'},
    {"log-level",   required_argument,  0, 'g'},
    {"inform",      no_argument,        0, 'I'},
    {NULL,          0,                  0, '\0'}
};

static const char *const HINTS[] =
{
    "Print the help message",
    "Print the man page",
    "Output the version of the software",
    "Enable debugging output",
    "Enable verbose output",
    "Specify how detailed logs should be. Possible values: 'error', 'warning', 'notice', 'info', 'verbose', 'debug'",
    "Enable basic information output",
    NULL
};

static int CFCheckHelpTopic(const char *topic)
{
    assert(topic != NULL);
    bool found = false;
    for (int i = 0; COMMANDS[i].name != NULL; ++i)
    {
        if (strcmp(COMMANDS[i].name, topic) == 0)
        {
            printf("Command:     %s\n", COMMANDS[i].name);
            printf("Usage:       %s\n", COMMANDS[i].usage);
            printf("Description: %s\n", COMMANDS[i].description);
            found = true;
            break;
        }
    }

    // Add more detailed explanation here if necessary:
    if (strcmp("help", topic) == 0)
    {
        printf("\nYou did it, you used the help command!\n");
    }
    else
    {
        if (!found)
        {
            printf("Unknown help topic: '%s'\n", topic);
            return EXIT_FAILURE;
        }
    }
    return EXIT_SUCCESS;
}

int main(int argc, const char *const *argv)
{
    if (StringEndsWith(argv[0], "lmdump"))
    {
        // Compatibility mode; act like lmdump if symlinked or renamed:
        return lmdump_main(argc, argv);
    }

    // When run separately it makes sense for cf-check to have INFO messages
    LogSetGlobalLevel(LOG_LEVEL_INFO);
    // In agent, NOTICE log level is default, and cf-check functions
    // will print less information

    if (argc < 2)
    {
        print_help();
        Log(LOG_LEVEL_ERR, "No command given");
        return EXIT_FAILURE;
    }

    int c = 0;
    int start_index = 1;
    const char *optstr = "+hMg:dvI"; // + means stop for non opt arg. :)
    while ((c = getopt_long(argc, (char *const *) argv, optstr, OPTIONS, &start_index))
            != -1)
    {
        switch (c)
        {
            case 'd':
            {
                LogSetGlobalLevel(LOG_LEVEL_DEBUG);
                break;
            }
            case 'v':
            {
                LogSetGlobalLevel(LOG_LEVEL_VERBOSE);
                break;
            }
            case 'I':
            {
                LogSetGlobalLevel(LOG_LEVEL_INFO);
                break;
            }
            case 'g':
            {
                LogSetGlobalLevelArgOrExit(optarg);
                break;
            }
            case 'V':
            {
                print_version();
                return EXIT_SUCCESS;
                break;
            }
            case 'h':
            {
                print_help();
                return EXIT_SUCCESS;
                break;
            }
            case 'M':
            {
                Writer *out = FileWriter(stdout);
                ManPageWrite(out, "cf-check", time(NULL),
                             CF_CHECK_SHORT_DESCRIPTION,
                             CF_CHECK_MANPAGE_LONG_DESCRIPTION,
                             OPTIONS, HINTS,
                             NULL, false,
                             true);
                FileWriterDetach(out);
                return EXIT_SUCCESS;
                break;
            }
            default:
            {
                return EXIT_FAILURE;
                break;
            }
        }
    }
    const char *const *const cmd_argv = argv + optind;
    int cmd_argc = argc - optind;
    const char *command = cmd_argv[0];

    if (StringSafeEqual_IgnoreCase(command, "lmdump"))
    {
        return lmdump_main(cmd_argc, cmd_argv);
    }
    if (StringSafeEqual_IgnoreCase(command, "dump"))
    {
        return dump_main(cmd_argc, cmd_argv);
    }
    if (StringSafeEqual_IgnoreCase(command, "diagnose"))
    {
        return diagnose_main(cmd_argc, cmd_argv);
    }
    if (StringSafeEqual_IgnoreCase(command, "backup"))
    {
        return backup_main(cmd_argc, cmd_argv);
    }
    if (StringSafeEqual_IgnoreCase(command, "repair") ||
        StringSafeEqual_IgnoreCase(command, "remediate"))
    {
        return repair_main(cmd_argc, cmd_argv);
    }
    if (StringSafeEqual_IgnoreCase(command, "help"))
    {
        if (cmd_argc > 2)
        {
            Log(LOG_LEVEL_ERR, "help takes exactly 0 or 1 arguments");
            return EXIT_FAILURE;
        }
        else if (cmd_argc <= 1)
        {
            print_help();
        }
        else
        {
            assert(cmd_argc == 2);
            CFCheckHelpTopic(cmd_argv[1]);
        }
        return EXIT_SUCCESS;
    }
    if (StringSafeEqual_IgnoreCase(command, "version"))
    {
        print_version();
        return EXIT_SUCCESS;
    }

    print_help();
    Log(LOG_LEVEL_ERR, "Unrecognized command: '%s'", command);
    return EXIT_FAILURE;
}

==> cfengine-3.12.2.tar.gz: cfengine-3.12.2/cf-check/cf-check.c <==
#include <lmdump.h>
#include <diagnose.h>
#include <backup.h>
#include <repair.h>
#include <string_lib.h>
#include <logging.h>

static void print_version()
{
    printf("cf-check BETA version %s\n", VERSION);
}

static void print_help()
{
    printf(
        "\n"
        "cf-check:\n"
        "\tUtility for diagnosis and repair of local CFEngine databases.\n"
        "\tThis BETA version of the tool is for testing purposes only.\n"
        "\n"
        "Commands:\n"
        "\tdump - Print the contents of a database file\n"
        "\tdiagnose - Assess the health of one or more database files\n"
        "\tbackup - Copy database files to a timestamped folder\n"
        "\trepair - Diagnose, then backup and delete any corrupt databases\n"
        "\tversion - Print version information\n"
        "\thelp - Print this help menu\n"
        "\n"
        "Usage:\n"
        "\t$ cf-check command [options]\n"
        "\n"
        "Examples:\n"
        "\t$ cf-check dump -a " WORKDIR "/state/cf_lastseen.lmdb\n"
        "\t$ cf-check diagnose\n"
        "\t$ cf-check repair\n"
        "\n");
}

int main(int argc, char **argv)
{
    if (StringEndsWith(argv[0], "lmdump"))
    {
        // Compatibility mode; act like lmdump if symlinked or renamed:
        return lmdump_main(argc, argv);
    }

    // When run separately it makes sense for cf-check to have INFO messages
    LogSetGlobalLevel(LOG_LEVEL_INFO);
    // In agent, NOTICE log level is default, and cf-check functions
    // will print less information

    if (argc < 2)
    {
        print_help();
        Log(LOG_LEVEL_ERR, "No command given");
        return 1;
    }

    const int cmd_argc = argc - 1;
    char **cmd_argv = argv + 1;
    char *command = cmd_argv[0];

    if (StringSafeEqual_IgnoreCase(command, "lmdump") ||
        StringSafeEqual_IgnoreCase(command, "dump"))
    {
        return lmdump_main(cmd_argc, cmd_argv);
    }
    if (StringSafeEqual_IgnoreCase(command, "diagnose"))
    {
        return diagnose_main(cmd_argc, cmd_argv);
    }
    if (StringSafeEqual_IgnoreCase(command, "backup"))
    {
        return backup_main(cmd_argc, cmd_argv);
    }
    if (StringSafeEqual_IgnoreCase(command, "repair") ||
        StringSafeEqual_IgnoreCase(command, "remediate"))
    {
        return repair_main(cmd_argc, cmd_argv);
    }

    if (StringSafeEqual_IgnoreCase(command, "help") ||
        StringSafeEqual_IgnoreCase(command, "--help") ||
        StringSafeEqual_IgnoreCase(command, "-h"))
    {
        print_help();
        return 0;
    }

    if (StringSafeEqual_IgnoreCase(command, "version") ||
        StringSafeEqual_IgnoreCase(command, "--version") ||
        StringSafeEqual_IgnoreCase(command, "-V"))
    {
        print_version();
        return 0;
    }

    print_help();
    Log(LOG_LEVEL_ERR, "Unrecognized command: '%s'", command);
    return 1;
}
